Option Strict Off
Imports System
Imports System.Collections.Generic
Imports NXOpen
Imports NXOpen.UF

Module Module1

    Dim theSession As Session = Session.GetSession()
    Dim theUfSession As UFSession = UFSession.GetUFSession
    Dim workPart As Part = theSession.Parts.Work
    Dim displayPart As Part = theSession.Parts.Display
    Dim startingWorkPart As Part = theSession.Parts.Work
    Dim lw As ListingWindow = theSession.ListingWindow

    Sub Main()

        If IsNothing(theSession.Parts.Work) Then
            'active part required
            Return
        End If

        lw.Open()

        Const undoMarkName As String = "NXJ journal"
        Dim markId1 As Session.UndoMarkId
        markId1 = theSession.SetUndoMark(Session.MarkVisibility.Visible, undoMarkName)

        Try
            Dim c As Assemblies.ComponentAssembly = displayPart.ComponentAssembly
            'to process the work part rather than the display part,
            '  comment the previous line and uncomment the following line
            'Dim c As ComponentAssembly = workPart.ComponentAssembly
            If Not IsNothing(c.RootComponent) Then
                '*** insert code to process 'root component' (assembly file)
                lw.WriteLine("Assembly: " & c.RootComponent.DisplayName)
                lw.WriteLine("")
                '*** end of code to process root component
                ReportComponentChildren(c.RootComponent, 0)
            Else
                '*** insert code to process piece part
                lw.WriteLine("Part has no components")
            End If
        Catch e As Exception
            theSession.ListingWindow.WriteLine("Failed: " & e.ToString)
        End Try

        lw.Close()

    End Sub

    Sub reportComponentChildren(ByVal comp As Assemblies.Component, _
    ByVal indent As Integer)

        For Each child As Assemblies.Component In comp.GetChildren()
            lw.WriteLine(New String(" ", indent * 2) & child.DisplayName() & " [" & child.Name & "]")

            EdgePoints(child)

            reportComponentChildren(child, indent + 1)
        Next
    End Sub

    Function FindEdge(ByVal theComp As Assemblies.Component, refSetName As String) As Edge

        Dim compBodies As New List(Of Body)

        MakeWorkPart(theComp)

        If refSetName.ToLower = "entire part" Then
            For Each tempBody As Body In workPart.Bodies
                If tempBody.IsSolidBody Then
                    compBodies.Add(tempBody)
                End If
            Next
        End If

        For Each theRefSet As ReferenceSet In workPart.GetAllReferenceSets

            'lw.WriteLine("wp ref set: " & theRefSet.Name)
            If theRefSet.Name = refSetName Then
                Dim refObjs() As NXObject = theRefSet.AskAllDirectMembers
                'lw.WriteLine(".AskAllDirectMembers count: " & refObjs.Length.ToString)
                For Each temp As NXObject In refObjs
                    'lw.WriteLine(temp.GetType.ToString)
                    If TypeOf (temp) Is Body Then
                        Dim theBody As Body = temp
                        If theBody.IsSolidBody Then
                            compBodies.Add(theBody)
                        End If
                    End If
                Next
            End If
        Next

        Dim i As Integer = 0
        Dim j As Integer = 0
        For Each tempBody As Body In compBodies
            i += 1
            'lw.WriteLine("body: " & i.ToString)
            Dim bodyEdges() As Edge = tempBody.GetEdges
            Dim myAttributeInfo As NXObject.AttributeInformation

            For Each tempEdge As Edge In bodyEdges
                j += 1
                'lw.WriteLine("edge: " & j.ToString)
                Try
                    myAttributeInfo = tempEdge.GetUserAttribute("test", NXObject.AttributeType.String, -1)

                    'attribute found
                    'lw.WriteLine("edge found")
                    AssemblyWorkPart()
                    Return tempEdge

                Catch ex As NXException
                    If ex.ErrorCode = 512008 Then
                        'attribute not found
                    Else
                        'unexpected error: show error message, undo, and exit journal
                        MsgBox(ex.ErrorCode & ": " & ex.Message)
                        Continue For
                    End If

                End Try
            Next

        Next

        AssemblyWorkPart()
        Return Nothing

    End Function

    Sub MakeWorkPart(ByVal theComponent As Assemblies.Component)

        Try
            Dim partLoadStatus1 As PartLoadStatus
            theSession.Parts.SetWorkComponent(theComponent, PartCollection.RefsetOption.Current, PartCollection.WorkComponentOption.Visible, partLoadStatus1)

            workPart = theSession.Parts.Work
            partLoadStatus1.Dispose()

        Catch ex As NXException

        End Try

    End Sub

    Sub EdgePoints(ByVal theComp As Assemblies.Component)

        Dim theEdge As Edge = FindEdge(theComp, theComp.ReferenceSet)
        If IsNothing(theEdge) Then
            Return
        End If

        Dim assemblyEdge As Edge = theComp.FindOccurrence(theEdge)

        Dim edgeEvaluator As System.IntPtr
        theUfSession.Eval.Initialize(assemblyEdge.Tag, edgeEvaluator)
        Dim lineInfo As UFEval.Line
        theUfSession.Eval.AskLine(edgeEvaluator, lineInfo)

        Dim stPt As Point3d
        Dim endPt As Point3d
        stPt.X = lineInfo.start(0)
        stPt.Y = lineInfo.start(1)
        stPt.Z = lineInfo.start(2)
        endPt.X = lineInfo.end(0)
        endPt.Y = lineInfo.end(1)
        endPt.Z = lineInfo.end(2)

        Dim edgeStPt As Point = workPart.Points.CreatePoint(stPt)
        edgeStPt.SetVisibility(SmartObject.VisibilityOption.Visible)

        Dim edgeEndPt As Point = workPart.Points.CreatePoint(endPt)
        edgeEndPt.SetVisibility(SmartObject.VisibilityOption.Visible)

        lw.WriteLine("edge start: " & stPt.ToString)
        lw.WriteLine("edge end: " & endPt.ToString)
        lw.WriteLine("")

    End Sub

    Sub AssemblyWorkPart()

        Dim nullAssemblies_Component As Assemblies.Component = Nothing

        Dim partLoadStatus1 As PartLoadStatus
        theSession.Parts.SetWorkComponent(nullAssemblies_Component, PartCollection.RefsetOption.Current, PartCollection.WorkComponentOption.Visible, partLoadStatus1)

        workPart = theSession.Parts.Work
        partLoadStatus1.Dispose()

    End Sub

    Public Function GetUnloadOption(ByVal dummy As String) As Integer

        'Unloads the image when the NX session terminates
        GetUnloadOption = NXOpen.Session.LibraryUnloadOption.AtTermination

        '----Other unload options-------
        'Unloads the image immediately after execution within NX
        'GetUnloadOption = NXOpen.Session.LibraryUnloadOption.Immediately

        'Unloads the image explicitly, via an unload dialog
        'GetUnloadOption = NXOpen.Session.LibraryUnloadOption.Explicitly
        '-------------------------------

    End Function

End Module
